﻿// FootnoteViewAdjusting.jsx

/*
    
Скрипт помогает подобрать два основных параметра оформления сносок: 
величину сдвига линейки над сноской и пространиство между последней строкой текста и первой строкой сноски.

Стандартно высота пространства сноски устанавливается равной произведению интерлиньяжа на число строк,
отсчёт от базовой линии последней строки сноски.
Это можно видеть, если установить нулевое смещение линейки (Параметры сносок > Макет : Смещение).

Но хочется воздуха над линейкой, отделяющей сноску от текста.
И в параметрах сноски высоту пространство над сноской определяет число в поле "Минимальная отбивка до первой сноски".
Это величина от верхнего края определённого выше пространства сноски до базовой линии последней строки текста, и это минимальное значение
Очевидно, что базовая линия последней строки текста над сноской всегда будет в пространстве интерлиньяжа текста.

Положение линейки над сноской (Параметры сносок > Макет : Смещение) тоже отсчитывается от высоты пространства сноски.

Скрипт сам устанавливает толщину линии 0,5 пт и длину линейки 20 мм (56.693 пт).

В последних версиях индизайна можно размещать сноски как под конкретной колонкой, так и на всю ширину многоколонника.
Это можно сделать как переопределением: Параметры текстового фрейма > Сноски : Включить переопределение,
так и в прежнем окне параметров сносок Текст > Параметры сносок документа.
Этот скрипт работает в предположении, что переопределение выключено, 
и все параметры определяются в окне Параметры сносок документа.

© Михаил Иванюшин, 2020,2021  |  dotextok@gmail.com  |  dotextok.ru

*/ 

#targetengine "FootnoteViewAdjustingShow"

var ProgressBar = function(title) { // ProgressBar
var w = new Window('palette', title, {x:0, y:0, width:700, height:40},{closeButton: true}), pb = w.add('progressbar', {x:20, y:12, width:660, height:12});       
  w.center();
this.reset = function(maxValue) {
  pb.value = 0;
  pb.maxvalue = maxValue;
  pb.visible = maxValue;
  w.show();
  w.update();
};
this.hit = function() {++pb.value; w.update();};
this.close = function() {w.close();};
} // ProgressBar
var pBar;

var programTitul = "Подбор отбивки сноски от текста";
app.scriptPreferences.userInteractionLevel = UserInteractionLevels.interactWithAll;
if (app.documents.length == 0) { 
    alert("Перед запуском скрипта откройте документ.",programTitul); 
    exit(); 
    }
var myDoc = app.activeDocument;
if (myDoc.footnoteOptions.footnoteTextStyle.name[0] == "[") {
    alert("В параметрах сносок документа, в позиции 'Стиль абзаца' должен быть указан стиль, определённый пользователем.",programTitul);  
    exit();    
    }
var leadselValue = true;
var leadtextValue = false;

var l2_value = false;
var l175_value = false;
var l15_value = false;
var l125_value = false;
var l1_value = true;

var leadtext_Value = "";
var lineShiftValue = 1;
var tmp;

var myOldXUnits = myDoc.viewPreferences.horizontalMeasurementUnits;
var myOldYUnits = myDoc.viewPreferences.verticalMeasurementUnits;
myDoc.viewPreferences.horizontalMeasurementUnits = MeasurementUnits.points;
myDoc.viewPreferences.verticalMeasurementUnits = MeasurementUnits.points;
var MainWinLocation = [-1,-1]; // положение на экране окна программы
var myDefSetName  = "FootnoteViewAdjusting.ini"; 
var myScriptFile = getScriptPath();
var myScriptFolder = decodeURI(myScriptFile.path);
var myFilePath = decodeURI(myScriptFolder + "/" + myDefSetName); 
var myDataFile = new File (myFilePath);
var myDataFileExists = false;
if (myDataFile.open("r")) { // такой файл есть
myDataFileExists = true;    
MainWinLocation[0] = myDataFile.readln();
MainWinLocation[1] = myDataFile.readln();
tmp = myDataFile.readln(); if (tmp == 1) leadselValue = true; else leadselValue = false;
tmp = myDataFile.readln(); if (tmp == 1) leadtextValue = true; else leadtextValue = false;
tmp = myDataFile.readln(); if (tmp == 1) l2_value = true; else l2_value = false;
tmp = myDataFile.readln(); if (tmp == 1) l175_value = true; else l175_value = false;
tmp = myDataFile.readln(); if (tmp == 1) l15_value = true; else l15_value = false;
tmp = myDataFile.readln(); if (tmp == 1) l125_value = true; else l125_value = false;
tmp = myDataFile.readln(); if (tmp == 1) l1_value = true; else l1_value = false;
leadtext_Value = myDataFile.readln();
lineShiftValue = myDataFile.readln();
myDataFile.close();
}
var myWin = myScriptWindow();
myWin.show();
///////////////////////////////////////////////////////
function myScriptWindow() { // myScriptWindow
var win = new Window ("palette", programTitul, undefined, {closeButton: true});
if (MainWinLocation[0] != -1) win.location = MainWinLocation;
win.alignChildren = ["fill", "fill"];
var fnOptions = myDoc.footnoteOptions;
var fPS = fnOptions.footnoteTextStyle;
var fPSpointSize = fPS.pointSize;
var footnotelineHeight;
if (fPS.leading == 1635019116) footnotelineHeight = fPSpointSize*fPS.autoLeading/100;
else footnotelineHeight = fPS.leading;
fnOptions.ruleLineWeight = 0.5;
fnOptions.ruleWidth = 56.693; // 20 mm

fnOptions.continuingRuleLineWeight = 0.5;
fnOptions.continuingRuleWidth = 56.693; 

var leadsel = win.add("radiobutton", undefined, "Отбивка сноски от текста в долях интерлиньяжа сноски");
///
leadsel.value = leadselValue;
leadsel.onClick = function() { // leadsel.onClick
test.enabled = true;
if (leadsel.value) leadtext.value = false;
leadbuttons.enabled = true;
lead.enabled = false;
} // leadsel.onClick
///
var leadbuttons = win.add("group");
leadbuttons.orientation = "row";
leadbuttons.alignChildren = ["right", "fill"];
var l2 = leadbuttons.add("radiobutton", undefined, "2,0  |");
var l175 = leadbuttons.add("radiobutton", undefined, "1,75  |");
var l15 = leadbuttons.add("radiobutton", undefined, "1,5  |");
var l125 = leadbuttons.add("radiobutton", undefined, "1,25  |");
var l1 = leadbuttons.add("radiobutton", undefined, "1,0");

l2.value = l2_value;
l175.value = l175_value;
l15.value = l15_value;
l125.value = l125_value;
l1.value = l1_value;

var leadvalue = win.add("group");
leadvalue.orientation = "row";
var leadtext = leadvalue.add("radiobutton", undefined, "Точное значение, пт");
var lead = leadvalue.add ("edittext {justify: 'center'}", undefined);
lead.enabled = false;
lead.characters = 6; 
lead.text = leadtext_Value;
leadtext.value = false;
lead.helpTip = "При указании отбивки для разделения целой и дробной частей числа используйте точку, а не запятую, единицу измерения вводить не надо.\rЛюбой знак в этом поле, отличный от цифры и точки, добавит в строку знак вопроса. Выделите букву, знак вопроса и сотрите. После этого можно продолжать ввод числа.";
///
var footLine = win.add("group");
footLine.orientation = "row"
footLine.add("statictext", undefined, "Смещение линейки над сносками, пт");
var lineShift = footLine.add ("edittext {justify: 'center'}", undefined);
lineShift.characters = 4; 
lineShift.text = lineShiftValue;
///
leadtext.onClick = function() { // leadtext.onClick
if (leadtext.value) leadsel.value = false;
leadbuttons.enabled = false;
lead.enabled = true;
///
var valid = /^[\d.]+$/.test (lead.text) && !isNaN(lead.text);
if (!valid) {
    lead.text = "?";
    test.enabled = false;
    return;
    }
else test.enabled = true;
} // leadtext.onClick
///
separator = win.add ("panel");
separator.minimumSize.height = separator.maximumSize.height = 1;
var buttons = win.add("group");
buttons.orientation = "row";
//buttons.alignChildren = ["left", "center"];
buttons.alignChildren = ["center", "bottom"];
var info = buttons.add("button", [0,0,28,28], "?");
var btnSize = [0, 0, 280, 30];
var test = buttons.add("button", btnSize, "Посмотреть, что получится");
///
lead.onChanging = function () { // lead.onChanging
var valid = /^[\d.]+$/.test (lead.text) && !isNaN(lead.text);
if (!valid) {
    lead.text = lead.text + "?";
    test.enabled = false;
    }
else test.enabled = true;
} // lead.onChanging
///
lineShift.onChanging = function () { // lineShift.onChanging
var valid = /^[-\d.]+$/.test (lineShift.text) && !isNaN(lineShift.text);
if (!valid) {
    lineShift.text = "?";
    test.enabled = false;
    return;
    }
else {
    test.enabled = true;
    fnOptions.footnoteFirstBaselineOffset= FootnoteFirstBaseline.LEADING_OFFSET;
    fnOptions.footnoteMinimumFirstBaselineOffset = 0; 
    fnOptions.ruleOffset = Number(lineShift.text); 
    fnOptions.continuingRuleOffset = Number(lineShift.text);     
    }
} // lineShift.onChanging
///
info.onClick = function () {
var myScriptFile = getScriptPath();
var folderPDFname = decodeURI(myScriptFile.path+ "/Info/");
var pdfList = Folder (folderPDFname).getFiles("*.pdf");
if (pdfList.length == 0) {
    alert ("В папке Info должен быть файл с информацией об этой программе, но он не найден.",programTitul);
   return;
    }
pdfList[0].execute();
} 
///
test.onClick = function () { // test.onClick
// все фреймы имеют метки MIN# или MAX#  тут # -- число строк в сноске    
// исходная высота каждого фрейма 127 пт
// она будет уменьшена до 77 пт, и затем будет по пункту увеличиваться.
// При этом будет контролироваться число строк.
// как только оно изменится первый раз -- это даст минимальное расстояние между текстом и сноской.
// Как изменится второй раз, то расстояние, что было до этого, даст максимальное расстояние.
    
var k;    
if (leadsel.value) { // leadsel.value 
    if (l2.value) { k = 2;};
    if (l175.value) { k = 1.75;};
    if (l15.value) { k = 1.5; };  
    if (l125.value) { k = 1.25; } ;    
    if (l1.value) { k = 1; };
    fnOptions.spacer = footnotelineHeight*k;
    } // leadsel.value 
if (leadtext.value) { // leadtext.value
    fnOptions.spacer = Number(lead.text);
    } // leadtext.value
///
if (leadsel.value == leadtext.value) {
    alert ("Ошибка в работе радиокнопок.",programTitul);
   return;    
    }
var tfN = myDoc.pages[0].textFrames.length;
var tFrames = myDoc.pages[0].textFrames.everyItem().getElements();
var min = [];
var max = [];
while (min.length > 0) min.shift();
while (max.length > 0) max.shift();
pBar = new ProgressBar(programTitul);
pBar.reset(tfN);
for (i = tfN-1; i >= 0; i--, pBar.hit()) { // i++  
    var cF = tFrames[i];
    if (cF.footnotes.length != 1) continue;
    var gb = cF.geometricBounds;
    yT = gb[0];
    yB = yT + 77;
    gb[2] =yB;
    cF.geometricBounds = gb;
    var curLabel = cF.label;
    var itIsMax;
    if (curLabel.match(/MIN/) == null) itIsMax = true; else itIsMax = false;
    var theIndex = Number(curLabel[3]);
    var delta;
    var story = cF.texts[0].parentStory;
    var lL = story.textContainers[0].lines.length; // (lineLength) число строк после уменьшения высоты фрейма
    var start = false;
    steps = 127-77;
    var rezNotFound = true;
    for (s = 0; s < steps; s++) { // s++
        yB = yB + 1;
        gb[2] =yB;
        cF.geometricBounds = gb;
        if (story.textContainers[0].lines.length == lL) continue;
        var textYstart = story.textContainers[0].lines[lL].characters[0].baseline; 
        if (itIsMax == false) { // !itIsMax
            // минимальная отбивка уже получена, осталось сохранить
            var footY = story.footnotes[0].characters[3].baseline;
            footY = footY - footnotelineHeight - fnOptions.ruleOffset; // это поправка, чтобы получить высоту пространства от базовой линии последней строки текста до линейки над сноской
            delta = Math.abs(footY - textYstart).toFixed(2);
            min[theIndex] = delta;
            rezNotFound = false;
            break;
            }  // !itIsMax        
        // появление новой строки -- это точка отсчёта
        if (start == false) {  
            start = true; 
             lL = story.textContainers[0].lines.length;
             continue;
            }
        // теперь надо уменьшить высоту на пункт, чтобы получить последнюю позицию, при которой отбивка была максимальной
        yB = yB - 1;
        gb[2] =yB;
        cF.geometricBounds = gb;   
        if (story.textContainers[0].lines.length != lL) {
            pBar.close();
            alert("Что-то пошло не так с расчётом максимальной отбивки",programTitul);
            return;
            }
        textYstart = story.textContainers[0].lines[lL-1].characters[0].baseline; 
        var footY = story.footnotes[0].characters[3].baseline;
        footY = footY - footnotelineHeight - fnOptions.ruleOffset; //  - footnotelineHeight - fnOptions.ruleOffset  это поправка, чтобы получить высоту пространства от базовой линии последней строки текста до линейки над сноской
        delta = Math.abs(footY - textYstart).toFixed(2);
        max[theIndex] = delta;  
        rezNotFound = false;        
        break;
        } // s++
    if (rezNotFound) {
        pBar.close();
        alert("Высота фрейма наращена до предела, а отбивки не вычислены.",programTitul);
        return;        
        }
    } //i++
pBar.close();
return;
} // test.onClick
/////
win.onClose = function() {
// что делать при щелчке на крестике в шапке окна программы    
myDataFile.open("w");
myDataFile.writeln(win.location[0]);
myDataFile.writeln(win.location[1]);
if (leadsel.value) myDataFile.writeln(1) else myDataFile.writeln(0);
if (leadtext.value) myDataFile.writeln(1) else myDataFile.writeln(0);
if (l2.value) myDataFile.writeln(1) else myDataFile.writeln(0);
if (l175.value) myDataFile.writeln(1) else myDataFile.writeln(0);
if (l15.value) myDataFile.writeln(1) else myDataFile.writeln(0);
if (l125.value) myDataFile.writeln(1) else myDataFile.writeln(0);
if (l1.value) myDataFile.writeln(1) else myDataFile.writeln(0);
myDataFile.writeln(lead.text); 
myDataFile.writeln(lineShift.text);
myDataFile.close();  
app.documents[0].viewPreferences.horizontalMeasurementUnits = myOldXUnits;
app.documents[0].viewPreferences.verticalMeasurementUnits = myOldYUnits;  
}
/////
return win;
}  // myScriptWindow
////
function myFile(myFileName) {
var myScriptFile = getScriptPath();
var myScriptFolder = decodeURI(myScriptFile.path);
var myFilePath = decodeURI(myScriptFolder + "/" +myFileName);
return myFilePath;
} //myFile
//
function getScriptPath() {
	try{
		return app.activeScript;
	}
	catch(myError){
		return File(myError.fileName);
	}
} //getScriptPath()
